@@ -99,32 +99,9 @@
        (removable-features
         (append non-target-features public-features)))
   (defconstant sb-impl:+internal-features+
-    ;;; Well, who would have guessed that our internal features list would nicely
-    ;;; repair damage induced by ASDF, namely: ASDF removes features when loaded.
-    ;;; Take a look at (DEFUN DETECT-OS) in uiop.lisp if you don't believe it,
-    ;;; and watch it in action after (require "ASDF") -
-    ;;;
-    ;;; * *FEATURES* =>
-    ;;; (:X86-64 :64-BIT :ANSI-CL :COMMON-LISP :ELF :GENCGC :HAIKU :IEEE-FLOATING-POINT
-    ;;; :LITTLE-ENDIAN :PACKAGE-LOCAL-NICKNAMES :SB-LDB
-    ;;; :SB-PACKAGE-LOCKS :SB-UNICODE :SBCL :UNIX)
-    ;;;
-    ;;; * (require :asdf)
-    ;;; ("ASDF" "asdf" "UIOP" "uiop")
-    ;;; * *FEATURES*
-    ;;; (:ASDF3.3 :ASDF3.2 :ASDF3.1 :ASDF3 :ASDF2 :ASDF :OS-UNIX
-    ;;; :NON-BASE-CHARS-EXIST-P :ASDF-UNICODE :X86-64 :64-BIT :ANSI-CL :COMMON-LISP
-    ;;; :ELF :GENCGC :IEEE-FLOATING-POINT :LITTLE-ENDIAN :PACKAGE-LOCAL-NICKNAMES
-    ;;; :SB-LDB :SB-PACKAGE-LOCKS :SB-UNICODE :SBCL :UNIX)
-    ;;;
-    ;;; So, what the heck happened to :HAIKU? It's gone.
-    ;;; Well this is pure evil. Just madness.
-    ;;; However, by stashing an extra copy of :HAIKU in the internal feature list,
-    ;;; we can stuff it back in because our contrib builder first loads ASDF
-    ;;; and then rebinds *FEATURES* with the union of the internal ones.
-    (append #+haiku '(:haiku)
             (remove-if (lambda (x) (member x #+sb-devel public-features
-                                             #-sb-devel removable-features)) *features*)))
+                                             #-sb-devel removable-features))
+                       *features*))
   (setq *features* (remove-if-not (lambda (x) (member x public-features))
                                   *features*)))
 
