@@ -229,7 +229,11 @@
       (destructuring-bind (name &optional initial-value (collector nil collectorp)
                                 &aux (n-value (copy-symbol name)))
           spec
-        (push `(,n-value ,(if (or initial-value collectorp) initial-value `(list nil)))
+        (push `(,n-value ,(if (or initial-value collectorp)
+                              initial-value
+                              `(#-sb-xc-host unaligned-dx-cons
+                                #+sb-xc-host list
+                                nil)))
               binds)
         (let ((macro-body
                (cond
@@ -352,6 +356,19 @@
              (setq ,val (pop ,tail))
              (progn ,@body)))))
 
+;;; Like GETHASH if HASH-TABLE contains an entry for KEY.
+;;; Otherwise, evaluate DEFAULT, store the resulting value in
+;;; HASH-TABLE and return two values: 1) the result of evaluating
+;;; DEFAULT 2) NIL.
+(defmacro ensure-gethash (key hash-table default)
+  (with-unique-names (n-key n-hash-table value foundp)
+    `(let ((,n-key ,key)
+           (,n-hash-table ,hash-table))
+       (multiple-value-bind (,value ,foundp) (gethash ,n-key ,n-hash-table)
+         (if ,foundp
+             (values ,value t)
+             (values (setf (gethash ,n-key ,n-hash-table) ,default) nil))))))
+
 ;;; (binding* ({(names initial-value [flag])}*) body)
 ;;; FLAG may be NIL or :EXIT-IF-NULL
 ;;;
@@ -594,7 +611,6 @@ NOTE: This interface is experimental and subject to change."
        (defun ,(symbolicate name "-CACHE-CLEAR") () (setq ,var-name nil))
        ',var-name)))
 
-(eval-when (:compile-toplevel :execute) ; leave this out of the core image
 (#+sb-xc-host defmacro #-sb-xc-host sb-xc:defmacro
  with-cache ((fun-name &rest actual-args) &body computation)
   (let* ((var-name (package-symbolicate (cl:symbol-package fun-name)
@@ -698,7 +714,7 @@ NOTE: This interface is experimental and subject to change."
                                                 `((incf (aref ,statistics-name 2))))
                                             idx1)))
                      ,entry)))
-           (values ,@result-temps)))))))
+           (values ,@result-temps))))))
 
 ;;; some syntactic sugar for defining a function whose values are
 ;;; cached by !DEFINE-HASH-CACHE
