@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!PRETTY")
+(in-package "SB-PRETTY")
 
 ;;;; user interface to the pretty printer
 
@@ -20,7 +20,6 @@
                                       (per-line-prefix nil per-line-prefix-p)
                                       (suffix ""))
                                 &body body)
-  #!+sb-doc
   "Group some output into a logical block. STREAM-SYMBOL should be either a
    stream, T (for *TERMINAL-IO*), or NIL (for *STANDARD-OUTPUT*). The printer
    control variable *PRINT-LEVEL* is automatically handled."
@@ -80,7 +79,6 @@
                                      ,@(if object (list object)))))))
 
 (defmacro pprint-exit-if-list-exhausted ()
-  #!+sb-doc
   "Cause the closest enclosing use of PPRINT-LOGICAL-BLOCK to return
    if its list argument is exhausted. Can only be used inside
    PPRINT-LOGICAL-BLOCK, and only when the LIST argument to
@@ -89,7 +87,6 @@
           PPRINT-LOGICAL-BLOCK."))
 
 (defmacro pprint-pop ()
-  #!+sb-doc
   "Return the next element from LIST argument to the closest enclosing
    use of PPRINT-LOGICAL-BLOCK, automatically handling *PRINT-LENGTH*
    and *PRINT-CIRCLE*. Can only be used inside PPRINT-LOGICAL-BLOCK.
@@ -106,6 +103,7 @@
 ;;;           the output buffer
 (deftype column ()
   '(and fixnum unsigned-byte))
+(deftype posn () 'fixnum)
 
 (defconstant initial-buffer-size 128)
 
@@ -119,7 +117,7 @@
 ;; for users to get a hold of the stream and stash it somewhere.
 ;; Anyway, just a thought... Maybe keep a small handful in a recyclable list?
 (defstruct (pretty-stream (:include ansi-stream
-                                    (out #'pretty-out)
+                                    (cout #'pretty-out)
                                     (sout #'pretty-sout)
                                     (misc #'pretty-misc))
                           (:constructor make-pretty-stream (target))
@@ -129,7 +127,7 @@
   ;; Line length we should format to. Cached here so we don't have to keep
   ;; extracting it from the target stream.
   (line-length (or *print-right-margin*
-                   (sb!impl::line-length target)
+                   (sb-impl::line-length target)
                    default-line-length)
                :type column
                :read-only t)
@@ -140,6 +138,9 @@
   (buffer (make-string initial-buffer-size) :type (simple-array character (*)))
   ;; The index into BUFFER where more text should be put.
   (buffer-fill-pointer 0 :type index)
+  ;; A list holding the positions in BUFFER of spaces which must not
+  ;; be removed if they are at the end of a line.
+  (buffer-significant-spaces nil :type list)
   ;; Whenever we output stuff from the buffer, we shift the remaining noise
   ;; over. This makes it difficult to keep references to locations in
   ;; the buffer. Therefore, we have to keep track of the total amount of
@@ -148,7 +149,7 @@
   ;; The column the first character in the buffer will appear in. Normally
   ;; zero, but if we end up with a very long line with no breaks in it we
   ;; might have to output part of it. Then this will no longer be zero.
-  (buffer-start-column (or (sb!impl::charpos target) 0) :type column)
+  (buffer-start-column (or (sb-impl::charpos target) 0) :type column)
   ;; The line number we are currently on. Used for *PRINT-LINES*
   ;; abbreviations and to tell when sections have been split across
   ;; multiple lines.
@@ -182,4 +183,5 @@
   (queue-tail nil :type list)
   (queue-head nil :type list)
   ;; Block-start queue entries in effect at the queue head.
-  (pending-blocks nil :type list))
+  (pending-blocks nil :type list)
+  (pending-blocks-length 0 :type index))
