@@ -220,6 +220,13 @@
   (:generator 2
     (load-foreign-symbol res foreign-symbol :dataref t)))
 
+#+sb-safepoint
+(defconstant thread-saved-csp-slot (- (1+ sb-vm::thread-header-slots)))
+
+(defconstant-eqx +destroyed-c-registers+
+  (loop for i from 0 to 18 collect i)
+  #'equal)
+
 (defun emit-c-call (vop nfp-save temp temp2 cfunc function)
   (let ((cur-nfp (current-nfp-tn vop)))
     (when cur-nfp
@@ -233,6 +240,10 @@
         (inst adr temp2 return)
         (inst stp cfp-tn temp2 (@ csp-tn))
         (storew-pair csp-tn thread-control-frame-pointer-slot temp thread-control-stack-pointer-slot thread-tn)
+        ;; OK to run GC without stopping this thread from this point
+        ;; on.
+        #+sb-safepoint
+        (storew csp-tn thread-tn thread-saved-csp-slot)
         (cond ((stringp function)
                (invoke-foreign-routine function cfunc))
               (t
@@ -241,16 +252,22 @@
                  (sap-stack
                   (load-stack-offset cfunc cur-nfp function)))
                (inst blr cfunc)))
-        (loop for reg in (list r0-offset r1-offset r2-offset r3-offset
-                               r4-offset r5-offset r6-offset r7-offset
-                               #-darwin r8-offset)
-              do
-              (inst mov
-                    (make-random-tn
-                     :kind :normal
-                     :sc (sc-or-lose 'descriptor-reg)
-                     :offset reg)
-                    0))
+        ;; Blank all boxed registers that potentially contain Lisp
+        ;; pointers, not just volatile ones, since GC could
+        ;; potentially observe stale pointers otherwise.
+        (loop for reg in (set-difference descriptor-regs
+                                         ;; any-reg temporaries contain no pointers
+                                         (list #-immobile-space r9-offset ;; invoke-foreign-routine doesn't touch it
+                                               r10-offset lexenv-offset))
+              do (inst mov
+                       (make-random-tn
+                        :kind :normal
+                        :sc (sc-or-lose 'descriptor-reg)
+                        :offset reg)
+                       0))
+        ;; No longer OK to run GC except at safepoints.
+        #+sb-safepoint
+        (storew zr-tn thread-tn thread-saved-csp-slot)
         (storew zr-tn thread-tn thread-control-stack-pointer-slot))
       return
       #-sb-thread
@@ -268,12 +285,7 @@
 
 (eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
   (defun destroyed-c-registers ()
-    (let ((gprs (list nl0-offset nl1-offset nl2-offset nl3-offset
-                      nl4-offset nl5-offset nl6-offset nl7-offset nl8-offset nl9-offset
-                      r0-offset r1-offset r2-offset r3-offset
-                      r4-offset r5-offset r6-offset r7-offset
-                      #-darwin r8-offset
-                      #-sb-thread r11-offset))
+    (let ((gprs +destroyed-c-registers+)
           (vars))
       (append
        (loop for gpr in gprs
@@ -293,7 +305,7 @@
   (:temporary (:sc any-reg :offset r9-offset
                :from (:argument 0) :to (:result 0)) cfunc)
   (:temporary (:sc control-stack :offset nfp-save-offset) nfp-save)
-  (:temporary (:sc any-reg :offset #+darwin r8-offset #-darwin r10-offset) temp)
+  (:temporary (:sc any-reg :offset r10-offset) temp)
   (:temporary (:sc any-reg :offset lexenv-offset) temp2)
   (:vop-var vop)
   (:generator 0
@@ -327,92 +339,11 @@
       (let ((delta (logandc2 (+ amount +number-stack-alignment-mask+)
                              +number-stack-alignment-mask+)))
         (inst add nsp-tn nsp-tn (add-sub-immediate delta))))))
-
-;;; long-long support
-;; (deftransform %alien-funcall ((function type &rest args) * * :node node)
-;;   (aver (sb-c:constant-lvar-p type))
-;;   (let* ((type (sb-c:lvar-value type))
-;;          (env (sb-c::node-lexenv node))
-;;          (arg-types (alien-fun-type-arg-types type))
-;;          (result-type (alien-fun-type-result-type type)))
-;;     (aver (= (length arg-types) (length args)))
-;;     (if (or (some (lambda (type)
-;;                     (and (alien-integer-type-p type)
-;;                          (> (sb-alien::alien-integer-type-bits type) 64)))
-;;                   arg-types)
-;;             (and (alien-integer-type-p result-type)
-;;                  (> (sb-alien::alien-integer-type-bits result-type) 64)))
-;;         (collect ((new-args) (lambda-vars) (new-arg-types))
-;;                  (loop with i = 0
-;;                        for type in arg-types
-;;                        for arg = (gensym)
-;;                        do
-;;                        (lambda-vars arg)
-;;                        (cond ((and (alien-integer-type-p type)
-;;                                    (> (sb-alien::alien-integer-type-bits type) 64))
-;;                               (when (oddp i)
-;;                                 ;; long-long is only passed in pairs of r0-r1 and r2-r3,
-;;                                 ;; and the stack is double-word aligned
-;;                                 (incf i)
-;;                                 (new-args 0)
-;;                                 (new-arg-types (parse-alien-type '(signed 8) env)))
-;;                               (incf i 2)
-;;                               (new-args `(logand ,arg #xffffffffffffffff))
-;;                               (new-args `(ash ,arg -64))
-;;                               (new-arg-types (parse-alien-type '(unsigned 64) env))
-;;                               (if (alien-integer-type-signed type)
-;;                                   (new-arg-types (parse-alien-type '(signed 64) env))
-;;                                   (new-arg-types (parse-alien-type '(unsigned 64) env))))
-;;                              (t
-;;                               (incf i (cond ((alien-double-float-type-p type)
-;;                                              0)
-;;                                             (t
-;;                                              1)))
-;;                               (new-args arg)
-;;                               (new-arg-types type))))
-;;                  (cond ((and (alien-integer-type-p result-type)
-;;                              (> (sb-alien::alien-integer-type-bits result-type) 64))
-;;                         (let ((new-result-type
-;;                                 (let ((sb-alien::*values-type-okay* t))
-;;                                   (parse-alien-type
-;;                                    (if (alien-integer-type-signed result-type)
-;;                                        '(values (unsigned 64) (signed 64))
-;;                                        '(values (unsigned 64) (unsigned 64)))
-;;                                    env))))
-;;                           `(lambda (function type ,@(lambda-vars))
-;;                              (declare (ignore type))
-;;                              (multiple-value-bind (low high)
-;;                                  (%alien-funcall function
-;;                                                  ',(make-alien-fun-type
-;;                                                     :arg-types (new-arg-types)
-;;                                                     :result-type new-result-type)
-;;                                                  ,@(new-args))
-;;                                (logior low (ash high 64))))))
-;;                        (t
-;;                         `(lambda (function type ,@(lambda-vars))
-;;                            (declare (ignore type))
-;;                            (%alien-funcall function
-;;                                            ',(make-alien-fun-type
-;;                                               :arg-types (new-arg-types)
-;;                                               :result-type result-type)
-;;                                            ,@(new-args))))))
-;;         (sb-c::give-up-ir1-transform))))
 
 ;;; Callback
 #-sb-xc-host
 (defun alien-callback-accessor-form (type sap offset)
-  (let ((parsed-type type))
-    (if (alien-integer-type-p parsed-type)
-        (let ((bits (sb-alien::alien-integer-type-bits parsed-type)))
-               (let ((byte-offset
-                      (cond ((< bits n-word-bits)
-                             (- n-word-bytes
-                                (ceiling bits n-byte-bits)))
-                            (t 0))))
-                 `(deref (sap-alien (sap+ ,sap
-                                          ,(+ byte-offset offset))
-                                    (* ,type)))))
-        `(deref (sap-alien (sap+ ,sap ,offset) (* ,type))))))
+  `(deref (sap-alien (sap+ ,sap ,offset) (* ,type))))
 
 #-sb-xc-host
 (defun alien-callback-assembler-wrapper (index result-type argument-types)
