@@ -11,7 +11,7 @@
 
 (in-package "SB-VM")
 
-(defconstant-eqx +fixup-kinds+ #(:absolute :cond-branch :uncond-branch :layout-id
+(defconstant-eqx +fixup-kinds+ #(:absolute :cond-branch :uncond-branch :layout-id :ubfm-imms
                                  :pc-relative :pc-relative-ldr-str :ldr-str :move-wide)
   #'equalp)
 
@@ -23,8 +23,7 @@
 (macrolet ((defreg (name offset)
              (let ((offset-sym (symbolicate name "-OFFSET")))
                `(progn
-                  (eval-when (:compile-toplevel :load-toplevel :execute)
-                    (defconstant ,offset-sym ,offset))
+                  (defconstant ,offset-sym ,offset)
                   (setf (svref *register-names* ,offset-sym) ,(symbol-name name)))))
 
            (defregset (name &rest regs)
@@ -41,7 +40,7 @@
   (defreg nl6 6)
   (defreg nl7 7)
   (defreg nl8 8)
-  (defreg nl9 9)
+  (defreg tmp 9)
 
   (defreg r0 10)
   (defreg r1 11)
@@ -54,11 +53,8 @@
   (defreg #-darwin r8 #+darwin reserved 18)
   (defreg r9 19)
 
-  (defreg #+darwin r8 #-darwin r10 20)
-  #+sb-thread
-  (defreg thread 21)
-  #-sb-thread
-  (defreg r11 21)
+  (defreg r10 20)
+  (defreg #+sb-thread thread #-sb-thread r11 21)
 
   (defreg lexenv 22)
 
@@ -67,7 +63,7 @@
   (defreg ocfp 25)
   (defreg cfp 26)
   (defreg csp 27)
-  (defreg tmp 28)
+  (defreg cardtable 28) ; preserved across C calls
   (defreg null 29)
   (defreg lr 30)
   (defreg nsp 31)
@@ -77,14 +73,15 @@
       null cfp nsp lr)
 
   (defregset descriptor-regs
-      r0 r1 r2 r3 r4 r5 r6 r7 r8 r9 #-darwin r10 #-sb-thread r11 lexenv)
+      r0 r1 r2 r3 r4 r5 r6 r7 #-darwin r8 r9 r10 #-sb-thread r11 lexenv)
 
+  ;; nl9 can't be selected by PACK as it is a freely usable temp reg
   (defregset non-descriptor-regs
-      nl0 nl1 nl2 nl3 nl4 nl5 nl6 nl7 nl8 nl9 nargs nfp ocfp lr)
+      nl0 nl1 nl2 nl3 nl4 nl5 nl6 nl7 nl8 nargs nfp ocfp lr)
 
   (defregset boxed-regs
       r0 r1 r2 r3 r4 r5 r6
-      r7 r8 r9 #-darwin r10 #-sb-thread r11 #+sb-thread thread lexenv)
+      r7 #-darwin r8 r9 r10 #-sb-thread r11 lexenv)
 
   ;; registers used to pass arguments
   ;;
@@ -93,8 +90,8 @@
   ;; names and offsets for registers used to pass arguments
   (defregset *register-arg-offsets*  r0 r1 r2 r3)
   (defparameter *register-arg-names* '(r0 r1 r2 r3))
-  (defregset *descriptor-args* r0 r1 r2 r3 r4 r5 r6 r7 r8 r9)
-  (defregset *non-descriptor-args* nl0 nl1 nl2 nl3 nl4 nl5 nl6 nl7 nl8 nl9)
+  (defregset *descriptor-args* r0 r1 r2 r3 r4 r5 r6 r7 #-darwin r8 r9 r10)
+  (defregset *non-descriptor-args* nl0 nl1 nl2 nl3 nl4 nl5 nl6 nl7 nl8)
   (defglobal *float-regs* (loop for i below 32 collect i)))
 
 
@@ -232,6 +229,7 @@
   (defregtn null descriptor-reg)
   (defregtn lexenv descriptor-reg)
   (defregtn tmp any-reg)
+  (defregtn cardtable any-reg)
 
   (defregtn nargs any-reg)
   (defregtn ocfp any-reg)
@@ -314,77 +312,6 @@
                  (complex-double-reg "Q"))
                offset)))))
 
-(defun combination-implementation-style (node)
-  (flet ((valid-funtype (args result)
-           (sb-c::valid-fun-use node
-                                (sb-c::specifier-type
-                                 `(function ,args ,result)))))
-    (case (sb-c::combination-fun-source-name node)
-      (logtest
-       (if (or (valid-funtype '(signed-word signed-word) '*)
-               (valid-funtype '(word word) '*)
-               (valid-funtype '(signed-word word) '*)
-               (valid-funtype '(word signed-word) '*))
-           (values :maybe nil)
-           (values :default nil)))
-      (logbitp
-       (cond
-         ((or (valid-funtype `((constant-arg (mod ,n-word-bits)) signed-word) '*)
-              (valid-funtype `((constant-arg (mod ,n-word-bits)) word) '*))
-          (values :direct nil))
-         (t (values :default nil))))
-      (%ldb
-       (flet ((validp (type)
-                (and (valid-funtype `((constant-arg (integer 1 ,(1- n-word-bits)))
-                                      (constant-arg integer)
-                                      ,type)
-                                    'unsigned-byte)
-                     (destructuring-bind (size posn integer)
-                         (sb-c::basic-combination-args node)
-                       (declare (ignore integer))
-                       (and (plusp (sb-c:lvar-value posn))
-                            (or (= (sb-c:lvar-value size) 1)
-                                (<= (+ (sb-c:lvar-value size)
-                                       (sb-c:lvar-value posn))
-                                    n-word-bits)))))))
-         (if (or (validp 'word)
-                 (validp 'signed-word))
-             (values :direct nil)
-             (values :default nil))))
-      (%dpb
-       (flet ((validp (type result-type)
-                (valid-funtype `(,type
-                                 (constant-arg (integer 1 ,(1- n-word-bits)))
-                                 (constant-arg (mod ,n-word-bits))
-                                 ,type)
-                               result-type)))
-         (if (and (or (validp 'signed-word 'signed-word)
-                      (validp 'word 'word))
-                  (not (destructuring-bind (new size posn integer) (sb-c::basic-combination-args node)
-                         (declare (ignore posn integer))
-                         (and (constant-lvar-p new)
-                              (let* ((size (lvar-value size))
-                                     (new (ldb (byte size 0) (lvar-value new))))
-                                (or (zerop new)
-                                    (= (logcount new) size)))))))
-             (values :direct nil)
-             (values :default nil))))
-      (signum
-       (if (or (valid-funtype '(signed-word) '*)
-               (valid-funtype '(word) '*))
-           (values :direct nil)
-           (values :default nil)))
-      (truncate
-       (destructuring-bind (n &optional d) (sb-c::basic-combination-args node)
-         (if (and d
-                  (constant-lvar-p d)
-                  (power-of-two-p (lvar-value d))
-                  (and (csubtypep (sb-c::lvar-type n) (specifier-type 'signed-word))
-                       (not (csubtypep (sb-c::lvar-type n) (specifier-type 'word)))))
-             (values :direct nil)
-             (values :default nil))))
-      (t (values :default nil)))))
-
 (defun primitive-type-indirect-cell-type (ptype)
   (declare (ignore ptype))
   nil)
@@ -411,6 +338,7 @@
         sb-arm64-asm::fixnum-add-sub-immediate-p
         sb-arm64-asm::encode-logical-immediate
         sb-arm64-asm::fixnum-encode-logical-immediate
+        fixnum-encode-logical-immediate-ignore-tag
         bic-encode-immediate
         bic-fixnum-encode-immediate
         logical-immediate-or-word-mask
