@@ -89,7 +89,7 @@
 ;;; Bind this to a stream to capture various internal debugging output.
 (defvar *compiler-trace-output* nil)
 ;;; These are the default, but the list can also include
-;;; :pre-ir2-optimize, :symbolic-asm.
+;;; :pre-ir2-optimize, :constraints.
 (defvar *compile-trace-targets* '(:ir1 :ir2 :vop :symbolic-asm :disassemble))
 (defvar *current-path*)
 (defvar *current-component*)
@@ -111,7 +111,7 @@
 ;;; In particular, ELF cores shrink the immobile code space down to just enough
 ;;; to contain all code, plus about 1/2 MiB of spare, which means that you can't
 ;;; subsequently compile a whole lot into immobile space.
-;;; The value is changed to :AUTO in make-target-2-load.lisp which supresses
+;;; The value is changed to :AUTO in make-target-2-load.lisp which suppresses
 ;;; codegen optimizations for immobile space, but nonetheless prefers to allocate
 ;;; the code there, falling back to dynamic space if there is no room left.
 ;;; These controls exist whether or not the immobile-space feature is present.
@@ -226,6 +226,13 @@
   (coverage-metadata nil :type (or (cons hash-table hash-table) null) :read-only t)
   (msan-unpoison nil :read-only t)
   (sset-counter 1 :type fixnum)
+  ;; Which GC the code generator should target. The codegen is basically the same for now
+  ;; (with tiny alterations) but it may be quite different eventually. And it would be
+  ;; great if COMPILE-FILE can be flexibile. If you have a compiler driver image that
+  ;; itself uses a particular GC, it should work to write FASLs for either GC as long as
+  ;; you don't load incompatible artifacts into the current image. Think of it as a
+  ;; compiler that just happens to be written in nearly-the-same implementation.
+  (allocator-target (default-gc-strategy) :type (member :gencgc :mark-region-gc))
   ;; if emitting a cfasl, the fasl stream to that
   (compile-toplevel-object nil :read-only t)
   ;; The current block compilation state.  These are initialized to
@@ -246,7 +253,7 @@
   ;; compiler to dump symbols in such a way that the loader can
   ;; reconstruct them in the correct package.
   (package-environment-changed nil :type boolean)
-  ;; Bidrectional map between IR1/IR2/assembler abstractions and a corresponding
+  ;; Bidirectional map between IR1/IR2/assembler abstractions and a corresponding
   ;; small integer or string identifier. One direction could be done by adding
   ;; the ID as slot to each object, but we want both directions.
   ;; These could just as well be scoped by WITH-IR1-NAMESPACE, but
