@@ -1895,7 +1895,8 @@
                                    ,',(if-vop-existsp (:translate %unary-ceiling)
                                                       `(truly-the fixnum (,(symbolicate '%unary- name) div))
                                                       `(%unary-truncate (truly-the ,fixnum-type quot)))
-                                   (,',unary-to-bignum quot))
+                                   (,',unary-to-bignum #+64-bit div
+                                                       #-64-bit quot))
                                (- number (* ,@(unless one-p
                                                 '(f-divisor))
                                             (+ quot
@@ -2100,6 +2101,25 @@
   (def <=)
   (def >=))
 
+(macrolet ((def (op)
+             `(deftransform ,op ((x y) ((integer #.most-negative-exactly-single-float-integer
+                                                 #.most-positive-exactly-single-float-integer)
+                                        float)
+                                 * :node node :important nil
+                                   :policy (> speed 1))
+                (unless (and (types-equal-or-intersect (lvar-type y) (specifier-type 'double-float))
+                             (types-equal-or-intersect (lvar-type y) (specifier-type 'single-float)))
+                  (give-up-ir1-transform))
+                (delay-ir1-transform node :ir1-phases)
+                `(if (single-float-p y)
+                     (,',op (%single-float x) (truly-the single-float y))
+                     (,',op (%double-float x) (truly-the double-float y))))))
+  (def =)
+  (def <)
+  (def >)
+  (def <=)
+  (def >=))
+
 (deftransform phase ((n))
   (splice-fun-args n 'complex 2)
   `(lambda (x y)
