@@ -32,9 +32,7 @@ otherwise evaluate ELSE and return its values. ELSE defaults to NIL."
          (then-block (ctran-starts-block then-ctran))
          (else-ctran (make-ctran))
          (else-block (ctran-starts-block else-ctran))
-         (node (make-if :test pred-lvar
-                        :consequent then-block
-                        :alternative else-block)))
+         (node (make-if pred-lvar then-block else-block)))
     ;; IR1-CONVERT-MAYBE-PREDICATE requires DEST to be CIF, so the
     ;; order of the following two forms is important
     (setf (lvar-dest pred-lvar) node)
@@ -99,9 +97,7 @@ otherwise evaluate ELSE and return its values. ELSE defaults to NIL."
       (multiple-value-bind (else-ctran else-block) (to-block else)
         (let* ((pred-ctran (make-ctran))
                (pred-lvar (make-lvar))
-               (node (make-if :test pred-lvar
-                              :consequent then-block
-                              :alternative else-block)))
+               (node (make-if pred-lvar then-block else-block)))
           (setf (lvar-dest pred-lvar) node)
           (ir1-convert start pred-ctran pred-lvar test)
           (link-node-to-previous-ctran node pred-ctran)
@@ -159,8 +155,7 @@ RETURN-FROM can be used to exit the form."
   (ctran-starts-block next)
   (let* ((dummy (make-ctran))
          (entry (make-entry))
-         (cleanup (make-cleanup :kind :block
-                                :mess-up entry)))
+         (cleanup (make-cleanup :block entry)))
     (push entry (lambda-entries (lexenv-lambda *lexenv*)))
     (setf (entry-cleanup entry) cleanup)
     (link-node-to-previous-ctran entry start)
@@ -187,8 +182,7 @@ extent of the block."
          (value-ctran (make-ctran))
          (value-lvar (make-lvar))
          (entry (first found))
-         (exit (make-exit :entry entry
-                          :value value-lvar)))
+         (exit (make-exit entry value-lvar)))
     (when (ctran-deleted-p exit-ctran)
       (throw 'locall-already-let-converted exit-ctran))
     (setf (lvar-dest value-lvar) exit)
@@ -246,8 +240,7 @@ STATEMENT must be a list. Other objects are illegal within the body."
   (let* ((dummy (make-ctran))
          (entry (make-entry))
          (segments (parse-tagbody statements))
-         (cleanup (make-cleanup :kind :tagbody
-                                :mess-up entry)))
+         (cleanup (make-cleanup :tagbody entry)))
     (push entry (lambda-entries (lexenv-lambda *lexenv*)))
     (setf (entry-cleanup entry) cleanup)
     (link-node-to-previous-ctran entry start)
@@ -284,7 +277,7 @@ constrained to be used only within the dynamic extent of the TAGBODY."
                     (compiler-error "attempt to GO to nonexistent tag: ~S"
                                     tag)))
          (entry (first found))
-         (exit (make-exit :entry entry)))
+         (exit (make-exit entry)))
     (when (ctran-deleted-p (second found))
       (throw 'locall-already-let-converted (second found)))
     (push exit (entry-exits entry))
@@ -653,7 +646,7 @@ Return VALUE without evaluating it."
                      thing))))
 
 (defun enclose (start next funs)
-  (let ((enclose (make-enclose :funs funs)))
+  (let ((enclose (make-enclose funs)))
     (link-node-to-previous-ctran enclose start)
     (use-ctran enclose next)
     (dolist (fun funs)
@@ -960,8 +953,7 @@ also processed as top level forms."
            (ctran-starts-block next)
            (let* ((enclose (ctran-use enclose-ctran))
                   (dynamic-extent (make-dynamic-extent))
-                  (cleanup (make-cleanup :kind :dynamic-extent
-                                         :mess-up dynamic-extent))
+                  (cleanup (make-cleanup :dynamic-extent dynamic-extent))
                   (dynamic-extent-ctran (make-ctran)))
              (setf (enclose-dynamic-extent enclose) dynamic-extent)
              (setf (dynamic-extent-cleanup dynamic-extent) cleanup)
@@ -1269,7 +1261,7 @@ care."
                   (leaf-type var))))
     (ir1-convert start dest-ctran dest-lvar `(the ,(type-specifier type)
                                                   ,value))
-    (let ((res (make-set :var var :value dest-lvar)))
+    (let ((res (make-set var dest-lvar)))
       (setf (lvar-dest dest-lvar) res)
       (cond (result ; SETQ with a result counts as a REF also
              (setf (leaf-ever-used var) t))
@@ -1306,8 +1298,7 @@ to TAG."
         (dummy2 (make-ctran)))
     (ir1-convert start dummy nil mess-up)
     (let* ((mess-node (ctran-use dummy))
-           (cleanup (make-cleanup :kind kind
-                                  :mess-up mess-node))
+           (cleanup (make-cleanup kind mess-node))
            (old-cup (lexenv-cleanup *lexenv*))
            (*lexenv* (make-lexenv :cleanup cleanup)))
       (setf (entry-cleanup (cleanup-mess-up old-cup)) cleanup)
@@ -1373,7 +1364,7 @@ the thrown values will be returned."
 #-c-stack-is-control-stack
 (def-ir1-translator restoring-nsp
     ((nsp &body body) start next result)
-  (let ((cleanup (make-cleanup :kind :restore-nsp))
+  (let ((cleanup (make-cleanup :restore-nsp))
         (nsp-ctran (make-ctran))
         (cleanup-ctran (make-ctran)))
     (ir1-convert start nsp-ctran nil nsp)
@@ -1549,7 +1540,7 @@ VALUES-FORM."
   (let* ((value-ctran (make-ctran))
          (forms-ctran (make-ctran))
          (value-lvar (make-lvar))
-         (delay (make-delay :value value-lvar)))
+         (delay (make-delay value-lvar)))
     (ctran-starts-block value-ctran)
     (ir1-convert start value-ctran value-lvar values-form)
     (ir1-convert-progn-body value-ctran forms-ctran nil forms)
