@@ -217,20 +217,19 @@
           (setf ftype (specifier-type `(function ,@(cddr name)))
                 where :declared))
         (make-global-var
-         :kind :global-function
-         :%source-name name
-         :type (if (or (eq where :declared)
-                       (and (not latep)
-                            (not notinline)
-                            *derive-function-types*))
-                   ftype
-                   (specifier-type 'function))
-         :defined-type (if (and (not latep) (not notinline))
-                           ftype
-                           (specifier-type 'function))
-         :where-from (if notinline
-                         where
-                         (maybe-defined-here name where)))))))
+         :global-function name
+         (if notinline
+             where
+             (maybe-defined-here name where))
+         (if (or (eq where :declared)
+                 (and (not latep)
+                      (not notinline)
+                      *derive-function-types*))
+             ftype
+             (specifier-type 'function))
+         (if (and (not latep) (not notinline))
+             ftype
+             (specifier-type 'function)))))))
 
 ;;; If NAME already has a valid entry in (FREE-FUNS *IR1-NAMESPACE*), then return
 ;;; the value. Otherwise, make a new GLOBAL-VAR using information from
@@ -254,16 +253,16 @@
                    (if (or expansion inlinep)
                        (let ((where (info :function :where-from name)))
                          (make-defined-fun
-                          :%source-name name
+                          name
+                          (if (and (eq inlinep 'notinline)
+                                   (neq where :declared))
+                              (specifier-type 'function)
+                              (global-ftype name))
+                          (if (eq inlinep 'notinline)
+                              where
+                              (maybe-defined-here name where))
                           :inline-expansion expansion
-                          :inlinep inlinep
-                          :where-from (if (eq inlinep 'notinline)
-                                          where
-                                          (maybe-defined-here name where))
-                          :type (if (and (eq inlinep 'notinline)
-                                         (neq where :declared))
-                                    (specifier-type 'function)
-                                    (global-ftype name))))
+                          :inlinep inlinep))
                        (find-global-fun name nil)))))))))
 
 ;;; Return the LEAF structure for the lexically apparent function
@@ -321,10 +320,7 @@
                  (let ((value (symbol-value name)))
                    (make-constant value (ctype-of value) name)))
                 (t
-                 (make-global-var :kind kind
-                                  :%source-name name
-                                  :type type
-                                  :where-from where-from)))))))
+                 (make-global-var kind name where-from type)))))))
 
 ;;; Return T if and only if OBJ's nature as an externalizable thing renders
 ;;; it a leaf for dumping purposes. Symbols are leaflike despite havings slots
@@ -380,6 +376,8 @@
           ((array t)
            (dotimes (i (array-total-size value))
              (grovel (row-major-aref value i))))
+          ;; Don't process and hold on to compiler data
+          ((or nlx-info sset-element))
           (instance
            ;; Behold the wonderfully clear sense of this-
            ;;  WHEN (EMIT-MAKE-LOAD-FORM VALUE)
@@ -655,9 +653,11 @@
   (aver-live-component *current-component*)
   ;; When FUNCTIONAL is of a type for which reanalysis isn't a trivial
   ;; no-op
-  (when (typep functional '(or optional-dispatch clambda))
-    (pushnew functional
-             (component-reanalyze-functionals *current-component*)))
+  (when (and (typep functional '(or optional-dispatch clambda))
+             (not (functional-reanalyze functional)))
+    (setf (functional-reanalyze functional) t)
+    (push functional
+          (component-reanalyze-functionals *current-component*)))
   functional)
 
 ;;; Generate a REF node for LEAF, frobbing the LEAF structure as
@@ -1136,9 +1136,7 @@
          (then-block (ctran-starts-block then-ctran))
          (else-ctran (make-ctran))
          (else-block (ctran-starts-block else-ctran))
-         (node (make-if :test pred-lvar
-                        :consequent then-block
-                        :alternative else-block)))
+         (node (make-if pred-lvar then-block else-block)))
     (setf (lvar-dest pred-lvar) node)
     (ir1-convert-combination-checking-type start pred-ctran pred-lvar form var)
     (link-node-to-previous-ctran node pred-ctran)
@@ -1417,11 +1415,11 @@
                    (specifier-type 'function)
                    (leaf-type var)))
          (res (make-defined-fun
-               :%source-name (leaf-source-name var)
-               :where-from (leaf-where-from var)
-               :type (if local-type
-                         (type-intersection local-type type)
-                         type)
+               (leaf-source-name var)
+               (if local-type
+                   (type-intersection local-type type)
+                   type)
+               (leaf-where-from var)
                :inlinep inlinep)))
     (when (defined-fun-p var)
       (setf (defined-fun-inline-expansion res)
@@ -1809,9 +1807,7 @@ possible.")
               name))
            found))
         (t
-         (make-global-var :kind :special
-                          :%source-name name
-                          :where-from :declared))))
+         (make-global-var :special name :declared))))
 
 (declaim (end-block))
 
@@ -1861,13 +1857,15 @@ possible.")
              (let ((kind (global-var-kind var)))
                (if (defined-fun-p var)
                    (make-defined-fun
-                    :%source-name name :type type :where-from :declared :kind kind
+                    name
+                    type
+                    :declared
+                    :kind kind
                     :inlinep (defined-fun-inlinep var)
                     :inline-expansion (defined-fun-inline-expansion var)
                     :same-block-p (defined-fun-same-block-p var)
                     :functional (defined-fun-functional var))
-                   (make-global-var :%source-name name :type type
-                                    :where-from :declared :kind kind))))
+                   (make-global-var kind name :declared type))))
        (when (defined-fun-p var)
          (let ((fun (defined-fun-functional var)))
            (when fun
@@ -1898,9 +1896,7 @@ possible.")
           (constant)
           (global-var
            (setf (gethash name free-vars)
-                 (make-global-var :%source-name name
-                                  :type type :where-from :declared
-                                  :kind (global-var-kind var)))))))))
+                 (make-global-var (global-var-kind var) name :declared type))))))))
 
 ;;; Similar in effect to FTYPE, but change the :INLINEP. Copying the
 ;;; global-var ensures that when we substitute a functional for a
@@ -1935,11 +1931,12 @@ possible.")
              ((:special :global))
              (:unknown
               (setf (gethash name free-vars)
-                    (make-global-var :%source-name name :type (leaf-type old)
-                                     :where-from (leaf-where-from old)
-                                     :kind (ecase kind
-                                             (special :special)
-                                             (global :global))))))))))
+                    (make-global-var (ecase kind
+                                       (special :special)
+                                       (global :global))
+                                     name
+                                     (leaf-where-from old)
+                                     (leaf-type old)))))))))
     ((start-block end-block)
      #-(and sb-devel sb-xc-host)
      (process-block-compile-proclamation kind args))
