@@ -9,7 +9,9 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
+
+(defconstant-eqx +fixup-kinds+ #(:absolute :jmp :lui :addi :sll-sa) #'equalp)
 
 
 ;;;; Registers
@@ -59,11 +61,12 @@
   (defreg csp 23) ; control stack pointer
   ;; More C unsaved temporaries.
   (defreg l1 24) ; tagged temporary 1
-  (defreg alloc 25) ; ALLOC pointer
-  ;; 26 and 27 are used by the system kernel.
-  ;; 28 is the global pointer of our C runtime, and used for
-  ;; jump/branch relaxation in Lisp.
-  (defreg gp 28)
+  (defreg cardbase 25)
+  ;; 26 and 27 are used by the syste kernel.
+  (defreg k0 26)
+  (defreg k1 27)
+  ;; 28 is the global pointer of our C runtime
+  (defreg gp-do-not-use 28)
   (defreg nsp 29) ; number (native) stack pointer
   ;; C frame pointer, or additional saved register.
   (defreg code 30) ; current function object
@@ -79,6 +82,10 @@
   (defregset *register-arg-offsets*
       a0 a1 a2 a3 a4 a5)
 
+  (defregset boxed-regs
+      a0 a1 a2 a3 a4 a5 fdefn lexenv
+      nfp ocfp lra l0 l1 code)
+
   (defregset reserve-descriptor-regs
       fdefn lexenv)
 
@@ -88,12 +95,14 @@
 
 ;;;; SB and SC definition:
 
+(!define-storage-bases
 (define-storage-base registers :finite :size 32)
 (define-storage-base float-registers :finite :size 32)
 (define-storage-base control-stack :unbounded :size 8)
 (define-storage-base non-descriptor-stack :unbounded :size 0)
 (define-storage-base constant :non-packed)
 (define-storage-base immediate-constant :non-packed)
+)
 
 (!define-storage-classes
 
@@ -266,7 +275,7 @@
   (defregtn bsp any-reg)
   (defregtn cfp any-reg)
   (defregtn csp any-reg)
-  (defregtn alloc any-reg)
+  (defregtn cardbase any-reg)
   (defregtn nsp any-reg)
 
   (defregtn code descriptor-reg)
@@ -277,33 +286,35 @@
 (defun immediate-constant-sc (value)
   (typecase value
     ((integer 0 0)
-     (sc-number-or-lose 'zero))
+     zero-sc-number)
     (null
-     (sc-number-or-lose 'null))
+     null-sc-number)
     (symbol
      (if (static-symbol-p value)
-         (sc-number-or-lose 'immediate)
+         immediate-sc-number
          nil))
-    ((or (integer #.sb!xc:most-negative-fixnum #.sb!xc:most-positive-fixnum)
-         character)
-     (sc-number-or-lose 'immediate))
+    ((signed-byte 30)
+     immediate-sc-number)
     #-sb-xc-host ; There is no such object type in the host
     (system-area-pointer
-     (sc-number-or-lose 'immediate))
+     immediate-sc-number)
     (character
-     (sc-number-or-lose 'immediate))))
+     immediate-sc-number)
+    (structure-object
+     (when (eq value sb-lockless:+tail+)
+       immediate-sc-number))))
 
 (defun boxed-immediate-sc-p (sc)
-  (or (eql sc (sc-number-or-lose 'zero))
-      (eql sc (sc-number-or-lose 'null))
-      (eql sc (sc-number-or-lose 'immediate))))
+  (or (eql sc zero-sc-number)
+      (eql sc null-sc-number)
+      (eql sc immediate-sc-number)))
 
 ;;;; Function Call Parameters
 
 ;;; The SC numbers for register and stack arguments/return values.
 ;;;
-(defconstant immediate-arg-scn (sc-number-or-lose 'any-reg))
-(defconstant control-stack-arg-scn (sc-number-or-lose 'control-stack))
+(defconstant immediate-arg-scn any-reg-sc-number)
+(defconstant control-stack-arg-scn control-stack-sc-number)
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
 
@@ -354,10 +365,6 @@
       (constant (format nil "Const~D" offset))
       (immediate-constant "Immed"))))
 
-(defun combination-implementation-style (node)
-  (declare (type sb!c::combination node) (ignore node))
-  (values :default nil))
-
 (defun primitive-type-indirect-cell-type (ptype)
   (declare (ignore ptype))
   nil)
