@@ -32,9 +32,8 @@
 #include "target-arch-os.h"
 #include "os.h"
 #include "globals.h"
-#include "dynbind.h"
 #include "genesis/cons.h"
-#include "genesis/fdefn.h"
+#include "genesis/symbol.h"
 #include "interr.h"
 #include "genesis/sap.h"
 #include "gc.h"
