@@ -223,12 +223,20 @@
         (setf (s1-kids b) (list k1 (make-s2 :id 'b-kid2 :parent b))
               (s1-kids c) (list k2)
               (s1-friends a) (list* b c (circular-list a))
-              (s1-friends b) (list a c)
+              (s1-friends b) (list c a)
               (s1-friends c) (list a b))
         (list a b c))))
 
 )) ; end EVAL-WHEN
 
+(defun load-form-is-default-mlfss-p (object)
+  (multiple-value-bind (creation-form init-form)
+      (make-load-form object)
+    (multiple-value-bind (ss-creation-form ss-init-form)
+        (make-load-form-saving-slots object)
+      (and (equal creation-form ss-creation-form)
+           (equal init-form ss-init-form)))))
+
 ;; Redefine the MAKE-LOAD-FORM method on FOO.
 (remove-method #'make-load-form (find-method #'make-load-form nil (list 'foo)))
 (defvar *foo-save-slots* nil)
@@ -241,13 +249,13 @@
   (let ((foo (make-foo :x 'x :y 'y)))
     (flet ((assert-canonical (slots)
              (let ((*foo-save-slots* slots))
-               (assert (sb-fasl:load-form-is-default-mlfss-p foo)))))
+               (assert (load-form-is-default-mlfss-p foo)))))
       (assert-canonical :all)
       (assert-canonical '(x y)) ; specifying all slots explicitly is still canonical
       (assert-canonical '(y x)))
     ;; specifying only one slot is not canonical
     (let ((*foo-save-slots* '(x)))
-      (assert (not (sb-fasl:load-form-is-default-mlfss-p foo))))))
+      (assert (not (load-form-is-default-mlfss-p foo))))))
 
 ;; A huge constant vector. This took 9 seconds to compile (on a MacBook Pro)
 ;; prior to the optimization for using fops to dump.
@@ -336,12 +344,14 @@
 
 ;; Track the make-load-form FOPs as they fly by at load-time.
 (defvar *call-tracker* nil)
+(defvar *fop-funs*
+  (car (ctu:find-code-constants #'sb-fasl::load-fasl-group :type '(simple-vector 128))))
 (dolist (fop-name '(sb-fasl::fop-instance))
-  (let* ((index (position fop-name sb-fasl::**fop-funs**
+  (let* ((index (position fop-name *fop-funs*
                           :key
                           (lambda (x) (and (functionp x) (sb-kernel:%fun-name x)))))
-         (fun (aref sb-fasl::**fop-funs** index)))
-    (setf (aref sb-fasl::**fop-funs** index)
+         (fun (aref *fop-funs* index)))
+    (setf (aref *fop-funs* index)
           (lambda (&rest args)
             (push fop-name *call-tracker*)
             (apply fun args)))))
@@ -543,3 +553,27 @@
 ;;; But thankfully we trap the read of the ID slot.
 (with-test (:name :illegal-typed-slot-value)
   (assert-error (spy-id *agent*)))
+
+(defun eq-cdr-constants-coalescing ()
+  (values
+   '(a (x . #1=(c)) #1# a)
+   '(b (x . #2=(c)) #2# b)))
+
+(with-test (:name :eq-cdr-constants-coalescing)
+  (multiple-value-bind (a b)
+      (funcall 'eq-cdr-constants-coalescing)
+    (assert (eq (cdadr a) (caddr a)))
+    (assert (eq (cdadr b) (caddr b)))))
+
+(with-test (:name :circularity-within-non-simple-array)
+  (let ((a #.(make-array 2 :adjustable t
+                           :initial-element '#1=(#1#))))
+    (assert (eq (aref (opaque-identity a) 0)
+                (car (aref (opaque-identity a) 0))))
+    (assert (eq (aref (opaque-identity a) 0)
+                (aref (opaque-identity a) 1)))))
+
+(with-test (:name :non-simple-array-constant-folding)
+  (let ((x #.(make-array 10 :fill-pointer 5)))
+    (assert (= (array-total-size x)
+               (array-total-size (opaque-identity x))))))
